#!/usr/bin/env wish

#******************************
# 
# osRename
# Version 2022-10-25
# Copyright 2022 Oliver Scholl < tcltk@posteo.de >
#
# Requirement:
# tcl 8.5 or higher
# tk 8.5 or higher
#
# Recommendation: none
# 
# Beschreibung des Programms:
# Das Programm ersetzt in allen Ordnern und Dateinamen unerwuenschte Zeichen 
# durch ein beliebiges Zeichen (z. B. einen Unterstrich). Das Programm steigt in alle Unterordner hinab,
# ist aber nicht rekursiv programmiert, um zu tiefe Verschachtelungen zu
# vermeiden.
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of 
# the license, or any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# merchantability or fitness for a particular purpose. 
# See the GNU General Public License for mor details.
#
#*************************************
#*** Generelle Einstellungen ***

set tclexecomp 0 ; # 1=wenn das Programm mit tclexecomp benutzt wird. Sonst 0.

package require msgcat

# Workaround fuer tclexecomp
namespace inscope :: [list ::source -encoding utf-8  [file join [file dirname [info script]] de.msg]]
namespace inscope :: [list ::source -encoding utf-8  [file join [file dirname [info script]] en.msg]]

font create GlobSchrift -family Helvetica -size 12 -slant roman -weight normal
ttk::style theme use default
ttk::style configure TButton -font GlobSchrift
ttk::style configure TLabel -font GlobSchrift
ttk::style configure TLabelFrame -font GlobSchrift 
ttk::style configure TMenubutton -font GlobSchrift 
ttk::style configure TRadiobutton -font GlobSchrift
ttk::style configure TEntry -font GlobSchrift 
ttk::style configure TCombobox -font GlobSchrift 
ttk::style configure TCheckbutton -font GlobSchrift
ttk::style configure TNotebook.Tab -font GlobSchrift 
ttk::style configure Treeview -font GlobSchrift 
ttk::style configure Treeview.Heading -font GlobSchrift
option add *TCombobox*Listbox.font GlobSchrift
option add *Dialog.msg.font GlobSchrift

set GlobKonf(Skriptname) [info script]
if {[file type $GlobKonf(Skriptname)] == "link"} {
	set GlobKonf(Skriptname) [file readlink $GlobKonf(Skriptname)]
}
::msgcat::mclocale de ;# Alternative Sprachen: de=deutsch en=englisch
::msgcat::mcload [file join [file dirname $GlobKonf(Skriptname)]]

if {$tclexecomp == 1} {
	set GlobKonf(Programmordner) [file join [file dirname [file normalize [info nameofexecutable]]]] 
	set GlobKonf(Konfigurationsdatei) [file join $GlobKonf(Programmordner) Konf.txt] 
} else {
	set GlobKonf(Programmordner) [file join [file dirname [file normalize [info script]]]] 
	set GlobKonf(Konfigurationsdatei) [file join $GlobKonf(Programmordner) Konf.txt] 
}

#*** Einstellungen zum Programm ***

set GlobKonf(ProgrammName) "osRename"
set GlobKonf(ProgrammVersion) "2022-10-25"
set GlobKonf(ProgrammTitel) "Dateien umbenennen"

set GlobKonf(Sprache) "deutsch"
set GlobKonf(Schriftgroesse) 12

set Startordner $env(HOME)
set Unterordner 1
set Typ 1
set Art 1
set Umlaute 1
set Zeichen "_"
set DateiZeichen ""
set MusterDateiZeichen "ErlaubteZeichen.txt"

if {[file exist $GlobKonf(Konfigurationsdatei)]} {
	set f [open $GlobKonf(Konfigurationsdatei) r]
	while {[gets $f Zeile] >= 0} {
		set Nummer [string rang $Zeile 0 0]
		set Wert [string rang $Zeile 2 end]
		switch $Nummer {
			1 {set GlobKonf(Sprache) $Wert}
			2 {set GlobKonf(Schriftgroesse) $Wert}
			3 {set Unterordner $Wert}
			4 {set Typ $Wert}
			5 {set Art $Wert}
			6 {set Umlaute $Wert}
			7 {set Zeichen $Wert}
			8 {set DateiZeichen $Wert}
		}
	}
	close $f
}

#*** Prozeduren ***

proc DateiZeichenErstellen {Ordner DateiZeichen} {
	# erstellt eine initiale Datei mit den zulaessigen Zeichen
	if {![file exist $DateiZeichen] && $DateiZeichen ne ""} {
		set ErlaubteZeichen {}
		lappend ErlaubteZeichen a b c d e f g h i j k l m n o p q r s t u v w x y z
		lappend ErlaubteZeichen A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
		lappend ErlaubteZeichen 1 2 3 4 5 6 7 8 9 0
		lappend ErlaubteZeichen ! \" § \$ \% \& \{ \} ( ) = + - _ ~ \# , . \;
		set f [open [file join $Ordner $DateiZeichen] w]
		foreach i $ErlaubteZeichen {
			puts $f $i
		}
		close $f
	}
}

proc DateienUmbenennen {Art Umlaute Zeichen Ordner DateiZeichen Simulation} {
	# Benennt die Dateien um.

	if { [catch {glob -nocomplain -directory $Ordner -types {f} *} tmp] == 0 } {
		foreach Element $tmp {
			set Name [file tail $Element]
			set NameNeu [Ersetzen $Name $Art $Umlaute $Zeichen $DateiZeichen]
			
			set ElementNeu [file join [file dirname $Element] $NameNeu]
			if {$ElementNeu ne $Element} {
				if {[file exist $ElementNeu]} {
					set Zaehler 1
					set DateiOhneEndung [file rootname $ElementNeu]	
					set DateiEndung [file extension $ElementNeu]						
					set ElementMitZaehler [string cat $DateiOhneEndung "_" $Zaehler $DateiEndung]
					while {[file exist $ElementMitZaehler]} {
						incr Zaehler
						set DateiOhneEndung [file rootname $ElementNeu]
						set DateiEndung [file extension $ElementNeu]
						set ElementMitZaehler [string cat $DateiOhneEndung "_" $Zaehler $DateiEndung]
					}
					set ElementNeu $ElementMitZaehler
				}
				if {$Simulation != 1} {
					catch {file rename $Element $ElementNeu}
				}
				.fr.frText.tx insert end "$Element\n"
				.fr.frText.tx insert end "$ElementNeu\n"
				.fr.frText.tx insert end "\n"
			}

		}
	}
}

proc Ersetzen {Name Art Umlaute Zeichen DateiZeichen} {
	# Ersetzt die Zeichen in dem Dateiname
	if {$Umlaute == 1} {
		set Liste {}
		lappend Liste "ä" "ae" "ö" "oe" "ü" "ue" "Ä" "Ae" "Ö" "Oe" "Ü" "Ue" "ß" "ss"	
		set Name [string map $Liste $Name]
	}

	set Laenge [string length $Name]
	switch $Art {
		1 {
			# Standard, auch Leerzeichen entfernen
			set ErlaubteZeichen {}
			lappend ErlaubteZeichen a b c d e f g h i j k l m n o p q r s t u v w x y z
			lappend ErlaubteZeichen A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
			lappend ErlaubteZeichen 1 2 3 4 5 6 7 8 9 0
			lappend ErlaubteZeichen ! \" § \$ \% \& \{ \} ( ) = + - _ ~ \# , . \;
		}
		2 {
			# Standard, Leerzeichen beibehalten
			set ErlaubteZeichen {}
			lappend ErlaubteZeichen a b c d e f g h i j k l m n o p q r s t u v w x y z
			lappend ErlaubteZeichen A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
			lappend ErlaubteZeichen 1 2 3 4 5 6 7 8 9 0
			lappend ErlaubteZeichen ! \" § \$ \% \& \{ \} ( ) = + - _ ~ \# , . \;
			lappend ErlaubteZeichen " "
		}
		3 {
			# Nur Leerzeichen
			set ErlaubteZeichen {}
			lappend ErlaubteZeichen " "
		}
		4 {
			# Individuell
			set ErlaubteZeichen {}
			set f [open $DateiZeichen r]
			while  {[gets $f Zeile] >= 0} {
				set j [string length $Zeile]
				for {set i 0} {$i < $j} {incr i} {
					 lappend ErlaubteZeichen [string index $Zeile $i]
				 }
			}
		}
	}

	set NameNeu ""
	for {set i 0} {$i < $Laenge} {incr i} {
		set Buchstabe [string index $Name $i]
		if {$Art != 3} {
			if {$Buchstabe in $ErlaubteZeichen} {
				set NameNeu [string cat $NameNeu $Buchstabe]
			} else {
				set NameNeu [string cat $NameNeu $Zeichen]
			}
		} else {
			if {$Buchstabe ni $ErlaubteZeichen} {
				set NameNeu [string cat $NameNeu $Buchstabe]
			} else {
				set NameNeu [string cat $NameNeu $Zeichen]
			}
		}
	}
	
	# $Zeichen nicht am Anfang und nicht am Ende
	set NameNeu [string trim $NameNeu $Zeichen]
	
	# mehrfache $Zeichen durch ein $Zeichen ersetzen
	while {[string first $Zeichen$Zeichen $NameNeu] >= 0} {
		set NameNeu [string map [list $Zeichen$Zeichen $Zeichen] $NameNeu]
	}
	
	# $Zeichen nicht vor oder nach einem Punkt
	set NameNeu [string map [list "${Zeichen}." "."] $NameNeu]
	set NameNeu [string map [list ".$Zeichen" "."] $NameNeu]
	
	
	return $NameNeu
}

proc Umbenennen {Typ Art Umlaute Zeichen Unterordner Startordner DateiZeichen Simulation} {
	# Benennt die Ordner und Dateien um.
	
	set Fehler 0

	if {$Fehler == 0 && $Startordner eq ""} {
		set Meldung [::msgcat::mc "Kein Startordner ausgewaehlt."]
		set Fehler 1
	}

	if {$Fehler == 0 && [string length $Zeichen] > 1} {
		set Meldung [::msgcat::mc "Unzulaessiges Ersetzungszeichen."]
		set Fehler 1
	}

	if {$Fehler == 0 && $Art == 4 && $DateiZeichen eq ""} {
		set Meldung [::msgcat::mc "Keine Ersetzungsdatei ausgewaehlt."]
		set Fehler 1
	}

	if {$Fehler == 0 && $Art == 4 && ![file exist $DateiZeichen]} {
		set Meldung [::msgcat::mc "Die Ersetzungsdatei existiert nicht."]
		set Fehler 1
	}

	# Text leeren
	.fr.frText.tx delete 1.0 end
	if {$Fehler == 0} {
		
		# Dateien umbenennen
		if {$Typ == 1 || $Typ == 3} {
			.fr.frText.tx insert end [::msgcat::mc "*** Dateien ***\n\n"]
			set ListeOrdner {}
			set AktuellerOrdner ""

			lappend ListeOrdner $Startordner
			set Index 0

			while {$Index < [llength $ListeOrdner]} {
				set AktuellerOrdner [lindex $ListeOrdner $Index]
				# Nicht die Ordner . bzw .. aufrufen
				if {[string index $AktuellerOrdner end] != "."} {
					if {[catch {glob -nocomplain -directory $AktuellerOrdner -types {d} *} tmp] == 0 } {
						foreach Ordner $tmp {
							DateienUmbenennen $Art $Umlaute $Zeichen $Ordner $DateiZeichen $Simulation
							if {$Unterordner == 1} {
								lappend ListeOrdner $Ordner
							}
						}
					}
				}
				incr Index
			}
			.fr.frText.tx insert end "\n"
		}
		
		# Ordner umbenennen
		if {$Typ == 1 || $Typ == 2} {
			.fr.frText.tx insert end [::msgcat::mc "*** Ordner ***\n\n"]
			set ListeOrdner {}
			set AktuellerOrdner ""

			lappend ListeOrdner $Startordner
			set Index 0

			while {$Index < [llength $ListeOrdner]} {
				set AktuellerOrdner [lindex $ListeOrdner $Index]
				# Nicht die Ordner . bzw .. aufrufen
				if {[string index $AktuellerOrdner end] != "."} {
					if {[catch {glob -nocomplain -directory $AktuellerOrdner -types {d} *} tmp] == 0 } {
						foreach Element $tmp {
							set Name [file tail $Element]
							set NameNeu [Ersetzen $Name $Art $Umlaute $Zeichen $DateiZeichen]
								
							set ElementNeu [file join [file dirname $Element] $NameNeu]
							if {$ElementNeu ne $Element} {
								if {[file exist $ElementNeu]} {
									set Zaehler 1
									set ElementMitZaehler [string cat $ElementNeu "_" $Zaehler]
									while {[file exist $ElementMitZaehler]} {
										incr Zaehler
										set ElementMitZaehler [string cat $ElementNeu "_" $Zaehler]
									}
									set ElementNeu $ElementMitZaehler
								}
								if {$Simulation != 1} {
									catch {file rename $Element $ElementNeu}
								}
								.fr.frText.tx insert end "$Element\n"
								.fr.frText.tx insert end "$ElementNeu\n"
								.fr.frText.tx insert end "\n"
							}

							if {$Unterordner == 1} {
								if {$Simulation != 1} {
									lappend ListeOrdner $ElementNeu
								} else {
									lappend ListeOrdner $Element
								}
							}
						}
					}
				}
				incr Index
			}
			.fr.frText.tx insert end "\n"
		}

	}
	
	if {$Fehler == 1} {
		.fr.frButton.btStart configure -state disabled
		tk_messageBox -title [::msgcat::mc "Fehler"] -message $Meldung -icon error -type ok
	} else {
		if {$Simulation != 1} {
			set Meldung "Umbenennung beendet."
		} else {
			set Meldung "Simulation beendet."
		}
		tk_messageBox -title [::msgcat::mc "Fertig"] -message $Meldung -icon info -type ok

		# Start-Button aktivieren
		.fr.frButton.btStart configure -state normal
	}
}

proc SpracheAendern {Sprache} {
	# stellt die Sprache ein
	global GlobKonf

	switch $Sprache {
		Deutsch {
			set GlobKonf(Sprache) "Deutsch"
			::msgcat::mclocale de
		}
		English {
			set GlobKonf(Sprache) "English"
			::msgcat::mclocale en
		}
	}
	::msgcat::mcload [file join [file dirname $GlobKonf(Skriptname)]]

	.m entryconfigure 0 -label [::msgcat::mc "Schliessen"]
	.m entryconfigure 1 -label [::msgcat::mc "Sprache"]
	.m entryconfigure 2 -label [::msgcat::mc "Schriftgroesse"]
	.m entryconfigure 3 -label [::msgcat::mc "Info"]

	.fr.lbOrdner configure -text [::msgcat::mc "Startordner:"]
	.fr.cbUnterordner configure -text [::msgcat::mc "inkl. Unterordner"] 
	.fr.frTyp configure -text [::msgcat::mc "Ordner/Dateien:"]
	.fr.frArt configure -text [::msgcat::mc "Ersetzung:"]
	.fr.cbUmlaute configure -text [::msgcat::mc "Umlaute ersetzen"]
	.fr.lbZeichen configure -text [::msgcat::mc "Ersetzen durch:"]
	.fr.frButton.btSimulation configure -text [::msgcat::mc "Simulation"]
	.fr.frButton.btStart configure -text [::msgcat::mc "Start"]
	.fr.frTyp.rbOrdnerDateien configure -text [::msgcat::mc "Ordner und Dateien aendern"]
	.fr.frTyp.rbOrdner configure -text [::msgcat::mc "Nur Ordner aendern"] 
	.fr.frTyp.rbDateien configure -text [::msgcat::mc "Nur Dateien aendern"]
	.fr.frArt.rbStandardInklusiveLeerzeichen configure -text [::msgcat::mc "Standard (auch Leerzeichen ersetzen)"]
	.fr.frArt.rbStandardOhneLeerzeichen configure -text [::msgcat::mc "Standard (Leerzeichen bestehen lassen)"]
	.fr.frArt.rbNurLeerzeichen configure -text [::msgcat::mc "nur Leerzeichen"]
	.fr.frArt.rbIndividuell configure -text [::msgcat::mc "individuell"]
	update idletasks
}

proc ProgrammBeenden {} {
	global GlobKonf
	global Unterordner
	global Typ
	global Art
	global Umlaute
	global Zeichen
	global DateiZeichen

	if {[catch {set f [open $GlobKonf(Konfigurationsdatei) w]}] == 0} {
		puts $f "1:$GlobKonf(Sprache)"
		puts $f "2:$GlobKonf(Schriftgroesse)"
		puts $f "3:$Unterordner"
		puts $f "4:$Typ"
		puts $f "5:$Art"
		puts $f "6:$Umlaute"
		puts $f "7:$Zeichen"
		puts $f "8:$DateiZeichen"
		close $f
	}
	exit
}

proc InfoAnzeigen {ProgrammName ProgrammTitel ProgrammVersion} {
	set Titel [::msgcat::mc "Info"]
	set Text "${ProgrammName}\nVersion ${ProgrammVersion}\n\nCopyright Oliver Scholl\nwww.tcltk.info\n
This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the license, or any later version.\n
This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of merchantability or fitness for a particular purpose. See the GNU General Public License for more details."
	tk_messageBox -title $Titel -message $Text -type ok
}

proc SchriftgroesseAendern {Schriftgroesse} {
	global GlobSchrift
	global GlobKonf
	font configure GlobSchrift -size $Schriftgroesse
	set GlobKonf(Schriftgroesse) $Schriftgroesse
}

# ***** GUI *****

wm protocol . WM_DELETE_WINDOW {
	# wird beim Schliessen des Hauptfensters ausgefuehrt
	ProgrammBeenden
}

font configure GlobSchrift -size $GlobKonf(Schriftgroesse)

wm title . $GlobKonf(ProgrammTitel)

option add *Menu.tearOff 0
. configure -menu .m
menu .m
.m add command -label [::msgcat::mc "Schliessen"] -font GlobSchrift -command ProgrammBeenden
.m add cascade -label [::msgcat::mc "Sprache"] -font GlobSchrift -menu .m.sprache
.m add cascade -label [::msgcat::mc "Schriftgroesse"] -font GlobSchrift -menu .m.schriftgroesse
.m add command -label [::msgcat::mc "Info"] -font GlobSchrift -command {InfoAnzeigen $GlobKonf(ProgrammName) $GlobKonf(ProgrammTitel) $GlobKonf(ProgrammVersion)}

menu .m.sprache
.m.sprache add radiobutton -label "Deutsch" -font GlobSchrift -variable Sprache -command {SpracheAendern $Sprache}
.m.sprache add radiobutton -label "English" -font GlobSchrift -variable Sprache -command {SpracheAendern $Sprache}

menu .m.schriftgroesse
.m.schriftgroesse add radiobutton -label 8 -variable Schriftgroesse -value 8 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 9 -variable Schriftgroesse -value 9 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 10 -variable Schriftgroesse -value 10 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 11 -variable Schriftgroesse -value 11 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 12 -variable Schriftgroesse -value 12 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 14 -variable Schriftgroesse -value 14 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 16 -variable Schriftgroesse -value 16 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 18 -variable Schriftgroesse -value 18 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 20 -variable Schriftgroesse -value 20 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}
.m.schriftgroesse add radiobutton -label 22 -variable Schriftgroesse -value 22 -underline 0 -font GlobSchrift -command {SchriftgroesseAendern $Schriftgroesse}

ttk::frame .fr
pack .fr -anchor nw -expand yes -fill both -padx 2 -pady 2
ttk::label .fr.lbOrdner -text [::msgcat::mc "Startordner:"]
ttk::entry .fr.enOrdner -width 50 -textvariable Startordner
ttk::button .fr.btOrdner -text "..." -command {.fr.frButton.btStart configure -state disabled; set Startordner [tk_chooseDirectory -initialdir $env(HOME)]}
ttk::checkbutton .fr.cbUnterordner -text [::msgcat::mc "inkl. Unterordner"] -variable Unterordner
ttk::labelframe .fr.frTyp -text [::msgcat::mc "Ordner/Dateien:"]
ttk::labelframe .fr.frArt -text [::msgcat::mc "Ersetzung:"]
ttk::checkbutton .fr.cbUmlaute -text [::msgcat::mc "Umlaute ersetzen"] -variable Umlaute
ttk::label .fr.lbZeichen -text [::msgcat::mc "Ersetzen durch:"]
ttk::entry .fr.enZeichen -width 3 -textvariable Zeichen
ttk::frame .fr.frButton
ttk::frame .fr.frText;# -borderwidth 2 -relief solid

grid .fr.lbOrdner -row 0 -column 0 -sticky w -padx 2 -pady 2
grid .fr.enOrdner -row 0 -column 1 -sticky nw -padx 2 -pady 2
grid .fr.btOrdner -row 0 -column 2 -sticky nw -padx 2 -pady 2
grid .fr.cbUnterordner -row 1 -column 0 -columnspan 3 -stick nw -padx 2 -pady 2
grid .fr.frTyp -row 2 -column 0 -columnspan 3 -sticky nwe -padx 2 -pady 20
grid .fr.frArt -row 3 -column 0 -columnspan 3 -sticky nwe -padx 2 -pady 20
grid .fr.cbUmlaute -row 4 -column 0 -columnspan 3 -stick nw -padx 2 -pady 2
grid .fr.lbZeichen -row 5 -column 0 -sticky w -padx 2 -pady 2
grid .fr.enZeichen -row 5 -column 1 -sticky nw -padx 2 -pady 2
grid .fr.frButton -row 6 -column 0 -columnspan 2 -sticky new -padx 2 -pady 15
grid .fr.frText -row 7 -column 0 -columnspan 3 -sticky nesw -padx 2 -pady 15 
grid rowconfigure .fr 7 -weight 1
grid columnconfigure .fr 2 -weight 1

ttk::button .fr.frButton.btSimulation -text [::msgcat::mc "Simulation"] -state normal -command {Umbenennen $Typ $Art $Umlaute $Zeichen $Unterordner $Startordner $DateiZeichen 1}
ttk::button .fr.frButton.btStart -text [::msgcat::mc "Start"] -state disabled -command {Umbenennen $Typ $Art $Umlaute $Zeichen $Unterordner $Startordner $DateiZeichen 0; .fr.frButton.btStart configure -state disabled}
grid .fr.frButton.btSimulation -row 0 -column 0  -padx 2 -pady 2
grid .fr.frButton.btStart -row 0 -column 1 -padx 2 -pady 2

ttk::radiobutton .fr.frTyp.rbOrdnerDateien -text [::msgcat::mc "Ordner und Dateien aendern"] -variable Typ -value 1
ttk::radiobutton .fr.frTyp.rbOrdner -text [::msgcat::mc "Nur Ordner aendern"] -variable Typ -value 2
ttk::radiobutton .fr.frTyp.rbDateien -text [::msgcat::mc "Nur Dateien aendern"] -variable Typ -value 3
grid .fr.frTyp.rbOrdnerDateien -row 0 -column 0 -columnspan 3 -sticky nw
grid .fr.frTyp.rbOrdner -row 1 -column 0 -columnspan 3 -sticky nw
grid .fr.frTyp.rbDateien -row 2 -column 0 -columnspan 3 -sticky nw

ttk::radiobutton .fr.frArt.rbStandardInklusiveLeerzeichen -text [::msgcat::mc "Standard (auch Leerzeichen ersetzen)"] -variable Art -value 1
ttk::radiobutton .fr.frArt.rbStandardOhneLeerzeichen -text [::msgcat::mc "Standard (Leerzeichen bestehen lassen)"] -variable Art -value 2
ttk::radiobutton .fr.frArt.rbNurLeerzeichen -text [::msgcat::mc "nur Leerzeichen"] -variable Art -value 3
ttk::radiobutton .fr.frArt.rbIndividuell -text [::msgcat::mc "individuell"] -variable Art -value 4
grid .fr.frArt.rbStandardInklusiveLeerzeichen -row 0 -column 0 -columnspan 3 -sticky nw
grid .fr.frArt.rbStandardOhneLeerzeichen -row 1 -column 0 -columnspan 3 -sticky nw
grid .fr.frArt.rbNurLeerzeichen -row 2 -column 0 -columnspan 3 -sticky nw
grid .fr.frArt.rbIndividuell -row 3 -column 0 -sticky nw

ttk::entry .fr.frArt.enDateiZeichen -width 50 -textvariable DateiZeichen
ttk::button .fr.frArt.btDateiZeichen -text "..." -command {.fr.frArt.rbIndividuell invoke; .fr.frButton.btStart configure -state disabled; set DateiZeichen [tk_getOpenFile -initialdir $env(HOME)]}
grid .fr.frArt.enDateiZeichen -row 3 -column 1 -sticky w -padx 2
grid .fr.frArt.btDateiZeichen -row 3 -column 2 -sticky nw -padx 2

text .fr.frText.tx -height 15 -width 15 -wrap none -xscrollcommand {.fr.frText.sbX set} -yscrollcommand {.fr.frText.sbY set}
ttk::scrollbar .fr.frText.sbX -orient horizontal -command {.fr.frText.tx xview}
ttk::scrollbar .fr.frText.sbY -command {.fr.frText.tx yview}
pack .fr.frText.sbX -side bottom -fill x
pack .fr.frText.sbY -side right -fill y
pack .fr.frText.tx -side top -expand yes -fill both

# Den Start-Button deaktivieren, wenn nach der Simulation nicht unmittelbar der Start-Button angeklickt wird
bind .fr.enOrdner <FocusIn> {.fr.frButton.btStart configure -state disabled}
bind .fr.cbUnterordner <Button-1> {.fr.frButton.btStart configure -state disabled}
bind .fr.frArt.rbStandardInklusiveLeerzeichen <Button-1> {.fr.frButton.btStart configure -state disabled}
bind .fr.frArt.rbStandardOhneLeerzeichen <Button-1> {.fr.frButton.btStart configure -state disabled}
bind .fr.frArt.rbNurLeerzeichen <Button-1> {.fr.frButton.btStart configure -state disabled}
bind .fr.frArt.rbIndividuell <Button-1> {.fr.frButton.btStart configure -state disabled}
bind .fr.frArt.enDateiZeichen <FocusIn> {.fr.frButton.btStart configure -state disabled; .fr.frArt.rbIndividuell invoke}

# ***** Nachdem die GUI erzeugt wurde *****

DateiZeichenErstellen $GlobKonf(Programmordner) $MusterDateiZeichen 
SpracheAendern $GlobKonf(Sprache)
